@@ -26,7 +26,7 @@ import pandas._libs.lib as lib
 import pathlib
 import re
 from collections import OrderedDict
-from pandas._typing import FilePathOrBuffer, StorageOptions
+from pandas._typing import CompressionOptions, FilePathOrBuffer, StorageOptions
 from typing import Union, IO, AnyStr, Sequence, Dict, List, Optional, Any
 
 from modin.error_message import ErrorMessage
@@ -56,6 +56,7 @@ def _read(**kwargs):
     from modin.data_management.factories.dispatcher import FactoryDispatcher
 
     Engine.subscribe(_update_engine)
+    squeeze = kwargs.pop("squeeze", False)
     pd_obj = FactoryDispatcher.read_csv(**kwargs)
     # This happens when `read_csv` returns a TextFileReader object for iterating through
     if isinstance(pd_obj, pandas.io.parsers.TextFileReader):
@@ -64,7 +65,10 @@ def _read(**kwargs):
             query_compiler=reader(*args, **kwargs)
         )
         return pd_obj
-    return DataFrame(query_compiler=pd_obj)
+    result = DataFrame(query_compiler=pd_obj)
+    if squeeze:
+        return result.squeeze(axis=1)
+    return result
 
 
 @_inherit_docstrings(pandas.read_csv)
@@ -550,8 +554,8 @@ def read_spss(
 @_inherit_docstrings(pandas.to_pickle)
 def to_pickle(
     obj: Any,
-    filepath_or_buffer: Union[str, pathlib.Path],
-    compression: Optional[str] = "infer",
+    filepath_or_buffer: FilePathOrBuffer,
+    compression: CompressionOptions = "infer",
     protocol: int = pickle.HIGHEST_PROTOCOL,
     storage_options: StorageOptions = None,
 ):
@@ -561,7 +565,11 @@ def to_pickle(
     if isinstance(obj, DataFrame):
         obj = obj._query_compiler
     return FactoryDispatcher.to_pickle(
-        obj, filepath_or_buffer, compression=compression, protocol=protocol
+        obj,
+        filepath_or_buffer=filepath_or_buffer,
+        compression=compression,
+        protocol=protocol,
+        storage_options=storage_options,
     )
 
 
@@ -629,6 +637,8 @@ def read_xml(
 
 @_inherit_docstrings(pandas.HDFStore)
 class HDFStore(pandas.HDFStore):
+    _return_modin_dataframe = True
+
     def __getattribute__(self, item):
         default_behaviors = ["__init__", "__class__"]
         method = super(HDFStore, self).__getattribute__(item)
@@ -667,7 +677,9 @@ class HDFStore(pandas.HDFStore):
                         for k, v in kwargs.items()
                     }
                     obj = super(HDFStore, self).__getattribute__(item)(*args, **kwargs)
-                    if isinstance(obj, pandas.DataFrame):
+                    if self._return_modin_dataframe and isinstance(
+                        obj, pandas.DataFrame
+                    ):
                         return DataFrame(obj)
                     return obj
 
