@@ -16,6 +16,7 @@ import numpy as np
 import pandas
 from pandas.errors import ParserWarning
 import pandas._libs.lib as lib
+from pandas.core.dtypes.common import is_list_like
 from collections import OrderedDict
 from modin.config import TestDatasetSize, Engine, Backend, IsExperimental
 from modin.utils import to_pandas
@@ -34,7 +35,6 @@ from .utils import (
     json_short_bytes,
     json_long_string,
     json_long_bytes,
-    eval_io,
     get_unique_filename,
     io_ops_bad_exc,
     eval_io_from_str,
@@ -45,6 +45,14 @@ from .utils import (
     generate_dataframe,
 )
 
+if Backend.get() == "Omnisci":
+    from modin.experimental.engines.omnisci_on_native.test.utils import (
+        eval_io,
+        align_datetime_dtypes,
+    )
+else:
+    from .utils import eval_io
+
 if Backend.get() == "Pandas":
     import modin.pandas as pd
 else:
@@ -127,12 +135,12 @@ def setup_excel_file(filename, row_size=NROWS, force=True):
         df.to_excel(filename)
 
 
-def setup_feather_file(filename, row_size=NROWS, force=True):
+def setup_feather_file(filename, row_size=NROWS, ncols=2, force=True):
     if os.path.exists(filename) and not force:
         pass
     else:
         df = pandas.DataFrame(
-            {"col1": np.arange(row_size), "col2": np.arange(row_size)}
+            {f"col{x + 1}": np.arange(row_size) for x in range(ncols)}
         )
         df.to_feather(filename)
 
@@ -339,55 +347,115 @@ class TestCsv:
             skipfooter=skipfooter,
         )
 
-    @pytest.mark.parametrize("true_values", [["Yes"], ["Yes", "true"], None])
-    @pytest.mark.parametrize("false_values", [["No"], ["No", "false"], None])
-    @pytest.mark.parametrize("skiprows", [2, lambda x: x % 2])
-    @pytest.mark.parametrize("skipfooter", [0, 10])
+    @pytest.mark.parametrize("header", ["infer", None, 0])
+    @pytest.mark.parametrize(
+        "skiprows",
+        [
+            2,
+            lambda x: x % 2,
+            lambda x: x > 25,
+            lambda x: x > 128,
+            np.arange(10, 50),
+            np.arange(10, 50, 2),
+        ],
+    )
     @pytest.mark.parametrize("nrows", [35, None])
-    @pytest.mark.parametrize("names", [["c1", "c2", "c3", "c4"], None])
+    @pytest.mark.parametrize(
+        "names",
+        [
+            [f"c{col_number}" for col_number in range(4)],
+            [f"c{col_number}" for col_number in range(6)],
+            None,
+        ],
+    )
+    @pytest.mark.parametrize("encoding", ["latin1", "windows-1251", None])
     def test_read_csv_parsing_2(
         self,
-        true_values,
-        false_values,
+        make_csv_file,
+        request,
+        header,
         skiprows,
-        skipfooter,
         nrows,
         names,
+        encoding,
     ):
-        xfail_case_1 = (
-            (false_values or true_values)
-            and Engine.get() != "Python"
-            and Backend.get() != "Omnisci"
-        )
-        xfail_case_2 = (
+        xfail_case = (
             Backend.get() == "Omnisci"
+            and header is not None
             and isinstance(skiprows, int)
             and names is None
-            and skipfooter == 0
             and nrows is None
-            and not true_values
-            and not false_values
         )
-        if xfail_case_1:
-            pytest.xfail("modin and pandas dataframes differs - issue #2446")
-        if xfail_case_2:
+        if xfail_case:
             pytest.xfail(
                 "read_csv fails because of duplicated columns names - issue #3080"
             )
+        if request.config.getoption(
+            "--simulate-cloud"
+        ).lower() != "off" and is_list_like(skiprows):
+            pytest.xfail(
+                reason="The reason of tests fail in `cloud` mode is unknown for now - issue #2340"
+            )
+        if encoding:
+            unique_filename = get_unique_filename()
+            make_csv_file(
+                filename=unique_filename,
+                encoding=encoding,
+            )
+        kwargs = {
+            "filepath_or_buffer": unique_filename
+            if encoding
+            else pytest.csvs_names["test_read_csv_regular"],
+            "header": header,
+            "skiprows": skiprows,
+            "nrows": nrows,
+            "names": names,
+            "encoding": encoding,
+        }
 
+        if Engine.get() != "Python":
+            df = pandas.read_csv(**dict(kwargs, nrows=1))
+            # in that case first partition will contain str
+            if df[df.columns[0]][df.index[0]] in ["c1", "col1", "c3", "col3"]:
+                pytest.xfail("read_csv incorrect output with float data - issue #2634")
         eval_io(
             fn_name="read_csv",
             check_exception_type=None,  # issue #2320
             raising_exceptions=None,
             check_kwargs_callable=not callable(skiprows),
             # read_csv kwargs
+            **kwargs,
+        )
+
+    @pytest.mark.parametrize("true_values", [["Yes"], ["Yes", "true"], None])
+    @pytest.mark.parametrize("false_values", [["No"], ["No", "false"], None])
+    @pytest.mark.parametrize("skipfooter", [0, 10])
+    @pytest.mark.parametrize("nrows", [35, None])
+    def test_read_csv_parsing_3(
+        self,
+        true_values,
+        false_values,
+        skipfooter,
+        nrows,
+    ):
+        xfail_case = (
+            (false_values or true_values)
+            and Engine.get() != "Python"
+            and Backend.get() != "Omnisci"
+        )
+        if xfail_case:
+            pytest.xfail("modin and pandas dataframes differs - issue #2446")
+
+        eval_io(
+            fn_name="read_csv",
+            check_exception_type=None,  # issue #2320
+            raising_exceptions=None,
+            # read_csv kwargs
             filepath_or_buffer=pytest.csvs_names["test_read_csv_yes_no"],
             true_values=true_values,
             false_values=false_values,
-            skiprows=skiprows,
             skipfooter=skipfooter,
             nrows=nrows,
-            names=names,
         )
 
     def test_read_csv_skipinitialspace(self):
@@ -404,18 +472,8 @@ class TestCsv:
     @pytest.mark.parametrize(
         "test_case",
         [
-            pytest.param(
-                "single_element",
-                marks=pytest.mark.xfail(
-                    reason="infinite recursion error - issue #2032"
-                ),
-            ),
-            pytest.param(
-                "single_column",
-                marks=pytest.mark.xfail(
-                    reason="infinite recursion error - issue #2032"
-                ),
-            ),
+            "single_element",
+            "single_column",
             "multiple_columns",
         ],
     )
@@ -891,12 +949,6 @@ class TestCsv:
                 StringIO(open(pytest.csvs_names["test_read_csv_regular"], "r").read())
             )
 
-        with pytest.warns(UserWarning):
-            pd.read_csv(
-                pytest.csvs_names["test_read_csv_regular"],
-                skiprows=lambda x: x in [0, 2],
-            )
-
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #2340",
@@ -908,6 +960,10 @@ class TestCsv:
             modin_warning=UserWarning,
             # read_csv kwargs
             filepath_or_buffer="https://raw.githubusercontent.com/modin-project/modin/master/modin/pandas/test/data/blah.csv",
+            # It takes about ~17Gb of RAM for Omnisci to import the whole table from this test
+            # because of too many (~1000) string columns in it. Taking a subset of columns
+            # to be able to run this test on low-RAM machines.
+            usecols=[0, 1, 2, 3] if Backend.get() == "Omnisci" else None,
         )
 
     @pytest.mark.parametrize("nrows", [21, 5, None])
@@ -1042,6 +1098,11 @@ class TestCsv:
             pytest.csvs_names["test_read_csv_regular"], method="modin"
         )
 
+        if Backend.get() == "Omnisci":
+            # Aligning DateTime dtypes because of the bug related to the `parse_dates` parameter:
+            # https://github.com/modin-project/modin/issues/3485
+            modin_df, pandas_df = align_datetime_dtypes(modin_df, pandas_df)
+
         df_equals(modin_df, pandas_df)
 
     @pytest.mark.parametrize(
@@ -1246,24 +1307,21 @@ class TestParquet:
             path="modin/pandas/test/data/hdfs.parquet",
         )
 
+    @pytest.mark.parametrize("path_type", ["url", "object"])
     @pytest.mark.xfail(
-        Engine.get() == "Python",
-        reason="S3-like path doesn't support in pandas with anonymous credentials. See issue #2301.",
+        condition="config.getoption('--simulate-cloud').lower() != 'off'",
+        reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
     )
-    def test_read_parquet_s3(self):
-        import s3fs
-
-        # Pandas currently supports only default credentials for boto therefore
-        # we use S3FileSystem with `anon=True` for  to make testing possible.
+    def test_read_parquet_s3(self, path_type):
         dataset_url = "s3://aws-roda-hcls-datalake/chembl_27/chembl_27_public_tissue_dictionary/part-00000-66508102-96fa-4fd9-a0fd-5bc072a74293-c000.snappy.parquet"
-        fs = s3fs.S3FileSystem(anon=True)
-        pandas_df = pandas.read_parquet(fs.open(dataset_url, "rb"))
-        modin_df_s3fs = pd.read_parquet(fs.open(dataset_url, "rb"))
-        df_equals(pandas_df, modin_df_s3fs)
+        if path_type == "object":
+            import s3fs
 
-        # Modin supports default and anonymous credentials and resolves this internally.
-        modin_df_s3 = pd.read_parquet(dataset_url)
-        df_equals(pandas_df, modin_df_s3)
+            fs = s3fs.S3FileSystem(anon=True)
+            with fs.open(dataset_url, "rb") as file_obj:
+                eval_io("read_parquet", path=file_obj)
+        else:
+            eval_io("read_parquet", path=dataset_url, storage_options={"anon": True})
 
     @pytest.mark.xfail(
         condition="config.getoption('--simulate-cloud').lower() != 'off'",
@@ -1625,9 +1683,10 @@ class TestHdf:
         reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
     )
     def test_HDFStore(self):
+        hdf_file = None
+        unique_filename_modin = get_unique_filename(extension="hdf")
+        unique_filename_pandas = get_unique_filename(extension="hdf")
         try:
-            unique_filename_modin = get_unique_filename(extension="hdf")
-            unique_filename_pandas = get_unique_filename(extension="hdf")
             modin_store = pd.HDFStore(unique_filename_modin)
             pandas_store = pandas.HDFStore(unique_filename_pandas)
 
@@ -1636,11 +1695,13 @@ class TestHdf:
             modin_store["foo"] = modin_df
             pandas_store["foo"] = pandas_df
 
-            assert assert_files_eq(unique_filename_modin, unique_filename_pandas)
             modin_df = modin_store.get("foo")
             pandas_df = pandas_store.get("foo")
             df_equals(modin_df, pandas_df)
 
+            modin_store.close()
+            pandas_store.close()
+            assert assert_files_eq(unique_filename_modin, unique_filename_pandas)
             assert isinstance(modin_store, pd.HDFStore)
 
             handle, hdf_file = tempfile.mkstemp(suffix=".hdf5", prefix="test_read")
@@ -1653,9 +1714,28 @@ class TestHdf:
             pandas_df = pandas.read_hdf(hdf_file, key="data/df1", mode="r")
             df_equals(modin_df, pandas_df)
         finally:
-            os.unlink(hdf_file)
+            if hdf_file:
+                os.unlink(hdf_file)
             teardown_test_files([unique_filename_modin, unique_filename_pandas])
 
+    @pytest.mark.xfail(
+        condition="config.getoption('--simulate-cloud').lower() != 'off'",
+        reason="The reason of tests fail in `cloud` mode is unknown for now - issue #3264",
+    )
+    def test_HDFStore_in_read_hdf(self):
+        filename = get_unique_filename(extension="hdf")
+        dfin = pd.DataFrame(np.random.rand(8, 8))
+        try:
+            dfin.to_hdf(filename, "/key")
+
+            with pd.HDFStore(filename) as h:
+                modin_df = pd.read_hdf(h, "/key")
+            with pandas.HDFStore(filename) as h:
+                pandas_df = pandas.read_hdf(h, "/key")
+            df_equals(modin_df, pandas_df)
+        finally:
+            teardown_test_files([filename])
+
 
 class TestSql:
     @pytest.mark.xfail(
@@ -2064,7 +2144,9 @@ class TestFeather:
     def test_read_feather(self):
         unique_filename = get_unique_filename(extension="feather")
         try:
-            setup_feather_file(filename=unique_filename)
+            # change the number of columns only if you know what you are doing;
+            # for details see https://github.com/modin-project/modin/pull/3465
+            setup_feather_file(filename=unique_filename, ncols=8)
 
             eval_io(
                 fn_name="read_feather",
